/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * ManagerEvent application -- send manager event
 * 
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/manager.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include <pthread.h>


static char *tdesc = "ManagerEvent Application";

static char *app = "ManagerEvent";

static char *synopsis = "Send manager event";

static char *descrip = 
"ManagerEvent(event):  Send manager event. Returns 0\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int managerevent_exec(struct ast_channel *chan, void *data)
{
	struct localuser *u;
	char info[512];
        char *eventname;
	char *eventbody;

	if (!data) {
		ast_log(LOG_WARNING, "ManagerEvent requires an argument (eventname|optional event body)\n");
		return -1;
	}

	strncpy(info, (char *)data, strlen((char *)data) + AST_MAX_EXTENSION-1);
	eventname = info;
	if (eventname) {
		eventbody = strchr(eventname, '|');
		if (eventbody) {
			*eventbody = '\0';
			eventbody++;
		}
	}

	LOCAL_USER_ADD(u);

	if(eventbody) {
            ast_log(LOG_DEBUG, "Sending manager event: %s, %s\n", eventname, eventbody);
            manager_event(EVENT_FLAG_CALL, eventname, 
			"Channel: %s\r\n%s\r\n",
			chan->name, eventbody);
	} else {
            ast_log(LOG_DEBUG, "Sending manager event: %s\n", eventname);
            manager_event(EVENT_FLAG_CALL, eventname, 
			"Channel: %s\r\n", chan->name);
	}

	LOCAL_USER_REMOVE(u);
	return 0;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

int load_module(void)
{
	return ast_register_application(app, managerevent_exec, synopsis, descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
